@@ -451,21 +451,22 @@ def start_new_series(cmdargs: argparse.Namespace) -> None:
         # create a default cover letter and store it where the strategy indicates
         cover = ('EDITME: cover title for %s' % seriesname,
                  '',
-                 '# Lines starting with # will be removed from the cover letter. You can use',
-                 '# them to add notes or reminders to yourself.',
+                 '# Lines starting with # will be removed from the cover letter. You can',
+                 '# use them to add notes or reminders to yourself.',
                  '',
-                 'EDITME: describe the purpose of this series. The information you put here',
-                 'will be used by the project maintainer to make a decision whether your',
-                 'patches should be reviewed, and in what priority order. Please be very',
-                 'detailed and link to any relevant discussions or sites that the maintainer',
-                 'can review to better understand your proposed changes. If you only have a',
-                 'single patch in your series, the contents of the cover letter will be',
-                 'appended to the "under-the-cut" portion of the patch.',
+                 'EDITME: describe the purpose of this series. The information you put',
+                 'here will be used by the project maintainer to make a decision whether',
+                 'your patches should be reviewed, and in what priority order. Please be',
+                 'very detailed and link to any relevant discussions or sites that the',
+                 'maintainer can review to better understand your proposed changes. If you',
+                 'only have a single patch in your series, the contents of the cover',
+                 'letter will be appended to the "under-the-cut" portion of the patch.',
                  '',
                  '# You can add trailers to the cover letter. Any email addresses found in',
-                 '# these trailers will be added to the addresses specified/generated during',
-                 '# the b4 send stage. You can also run "b4 prep --auto-to-cc" to auto-populate',
-                 '# the To: and Cc: trailers based on the code being modified.',
+                 '# these trailers will be added to the addresses specified/generated',
+                 '# during the b4 send stage. You can also run "b4 prep --auto-to-cc" to',
+                 '# auto-populate the To: and Cc: trailers based on the code being',
+                 '# modified.',
                  '',
                  'Signed-off-by: %s <%s>' % (usercfg.get('name', ''), usercfg.get('email', '')),
                  '',
@@ -480,7 +481,7 @@ def start_new_series(cmdargs: argparse.Namespace) -> None:
         if revision is None:
             revision = 1
         prefixes = list()
-        if cmdargs.set_prefixes and len(prefixes[0].strip()):
+        if cmdargs.set_prefixes:
             prefixes = list(cmdargs.set_prefixes)
         else:
             config = b4.get_main_config()
@@ -686,17 +687,21 @@ def edit_cover() -> None:
     corecfg = b4.get_config_from_git(r'core\..*', {'editor': os.environ.get('EDITOR', 'vi')})
     editor = corecfg.get('editor')
     logger.debug('editor=%s', editor)
-    # We give it a suffix .rst in hopes that editors autoload restructured-text rules
-    with tempfile.NamedTemporaryFile(suffix='.rst') as temp_cover:
-        temp_cover.write(cover.encode())
-        temp_cover.seek(0)
+    # Use COMMIT_EDITMSG name in hopes that editors autoload git commit rules
+    with tempfile.TemporaryDirectory(prefix='b4-') as temp_dir:
+        temp_fpath = os.path.join(temp_dir, 'COMMIT_EDITMSG')
+        with open(temp_fpath, 'xb') as temp_cover:
+            temp_cover.write(cover.encode())
+
         sp = shlex.shlex(editor, posix=True)
         sp.whitespace_split = True
-        cmdargs = list(sp) + [temp_cover.name]
+        cmdargs = list(sp) + [temp_fpath]
         logger.debug('Running %s' % ' '.join(cmdargs))
         sp = subprocess.Popen(cmdargs)
         sp.wait()
-        new_cover = temp_cover.read().decode(errors='replace').strip()
+
+        with open(temp_fpath, 'rb') as temp_cover:
+            new_cover = temp_cover.read().decode(errors='replace').strip()
 
     if new_cover == cover:
         logger.info('Cover letter unchanged.')
@@ -1001,7 +1006,7 @@ def make_msgid_tpt(change_id: str, revision: str, domain: Optional[str] = None)
     return msgid_tpt
 
 
-def get_cover_dests(cbody: str, hide: bool = True) -> Tuple[List, List, str]:
+def get_cover_dests(cbody: str) -> Tuple[List, List, str]:
     htrs, cmsg, mtrs, basement, sig = b4.LoreMessage.get_body_parts(cbody)
     tos = list()
     ccs = list()
@@ -1012,8 +1017,7 @@ def get_cover_dests(cbody: str, hide: bool = True) -> Tuple[List, List, str]:
         elif mtr.lname == 'cc':
             ccs.append(mtr.addr)
             mtrs.remove(mtr)
-    if hide:
-        cbody = b4.LoreMessage.rebuild_message(htrs, cmsg, mtrs, basement, sig)
+    cbody = b4.LoreMessage.rebuild_message(htrs, cmsg, mtrs, basement, sig)
     return tos, ccs, cbody
 
 
@@ -1113,8 +1117,7 @@ def get_mailfrom() -> Tuple[str, str]:
     return usercfg.get('name'), usercfg.get('email')
 
 
-def get_prep_branch_as_patches(movefrom: bool = True, thread: bool = True,
-                               addtracking: bool = True, hide_cover_to_cc: bool = False
+def get_prep_branch_as_patches(movefrom: bool = True, thread: bool = True, addtracking: bool = True
                                ) -> Tuple[List, List, str, List[Tuple[str, email.message.Message]]]:
     cover, tracking = load_cover(strip_comments=True)
 
@@ -1176,10 +1179,10 @@ def get_prep_branch_as_patches(movefrom: bool = True, thread: bool = True,
     ztracking = gzip.compress(bytes(json.dumps(tracking), 'utf-8'))
     b64tracking = base64.b64encode(ztracking).decode()
     # A little trick for pretty wrapping
-    wrapped = textwrap.wrap('X-B4-Tracking: v=1; b=' + b64tracking, width=75)
-    thdata = ' '.join(wrapped).replace('X-B4-Tracking: ', '')
+    wrapped = textwrap.wrap('X-B4-Tracking: v=1; b=' + b64tracking, subsequent_indent=' ', width=75)
+    thdata = ''.join(wrapped).replace('X-B4-Tracking: ', '')
 
-    alltos, allccs, cbody = get_cover_dests(cover_letter, hide=hide_cover_to_cc)
+    alltos, allccs, cbody = get_cover_dests(cover_letter)
     if len(patches) == 1:
         mixin_cover(cbody, patches)
     else:
@@ -1192,8 +1195,7 @@ def get_prep_branch_as_patches(movefrom: bool = True, thread: bool = True,
     return alltos, allccs, tag_msg, patches
 
 
-def get_sent_tag_as_patches(tagname: str, revision: int, hide_cover_to_cc: bool = False
-                            ) -> Tuple[List, List, List[Tuple[str, email.message.Message]]]:
+def get_sent_tag_as_patches(tagname: str, revision: int) -> Tuple[List, List, List[Tuple[str, email.message.Message]]]:
     cover, base_commit, change_id = get_base_changeid_from_tag(tagname)
 
     csubject, cbody = get_cover_subject_body(cover)
@@ -1210,7 +1212,7 @@ def get_sent_tag_as_patches(tagname: str, revision: int, hide_cover_to_cc: bool
                                       seriests=seriests,
                                       mailfrom=mailfrom)
 
-    alltos, allccs, cbody = get_cover_dests(cbody, hide=hide_cover_to_cc)
+    alltos, allccs, cbody = get_cover_dests(cbody)
     if len(patches) == 1:
         mixin_cover(cbody, patches)
     else:
@@ -1251,8 +1253,6 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
     mybranch = b4.git_get_current_branch()
 
     config = b4.get_main_config()
-    if not cmdargs.hide_cover_to_cc and config.get('send-hide-cover-to-cc', '').lower() in {'yes', 'true', '1'}:
-        cmdargs.hide_cover_to_cc = True
 
     tag_msg = None
     cl_msgid = None
@@ -1264,8 +1264,7 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
             sys.exit(1)
 
         try:
-            todests, ccdests, patches = get_sent_tag_as_patches(tagname, revision=revision,
-                                                                hide_cover_to_cc=cmdargs.hide_cover_to_cc)
+            todests, ccdests, patches = get_sent_tag_as_patches(tagname, revision=revision)
         except RuntimeError as ex:
             logger.critical('CRITICAL: Failed to convert tag to patches: %s', ex)
             sys.exit(1)
@@ -1282,9 +1281,14 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
             logger.info('---')
             sys.exit(1)
 
+        status = b4.git_get_repo_status()
+        if len(status):
+            logger.critical('CRITICAL: Repository contains uncommitted changes.')
+            logger.critical('          Stash or commit them first.')
+            sys.exit(1)
+
         try:
-            todests, ccdests, tag_msg, patches = get_prep_branch_as_patches(
-                hide_cover_to_cc=cmdargs.hide_cover_to_cc)
+            todests, ccdests, tag_msg, patches = get_prep_branch_as_patches()
         except RuntimeError as ex:
             logger.critical('CRITICAL: Failed to convert range to patches: %s', ex)
             sys.exit(1)
@@ -1314,13 +1318,11 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
                     continue
                 if btr.addr[1] in seen:
                     continue
-                if commit and btr.lname == 'cc':
-                    # CC's in individual patches don't get added to global Cc's,
-                    # we use the pcss dict to track them.
+                if commit:
                     if commit not in pccs:
                         pccs[commit] = list()
-                    pccs[commit].append(btr.addr)
-                    # Doesn't get added to seen, in case a later patch puts it into global
+                    if btr.addr not in pccs[commit]:
+                        pccs[commit].append(btr.addr)
                     continue
                 seen.add(btr.addr[1])
                 if btr.lname == 'to':
@@ -1381,15 +1383,20 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
         pathlib.Path(cmdargs.output_dir).mkdir(parents=True, exist_ok=True)
 
     sconfig = b4.get_sendemail_config()
-    endpoint = config.get('send-endpoint-web', '')
-    if not re.search(r'^https?://', endpoint):
-        endpoint = None
-    if not endpoint and not sconfig.get('smtpserver'):
-        # Use the default endpoint if we are in the kernel repo
-        topdir = b4.git_get_toplevel()
-        if os.path.exists(os.path.join(topdir, 'Kconfig')):
-            logger.debug('No sendemail configs found, will use the default web endpoint')
-            endpoint = DEFAULT_ENDPOINT
+    # If we have an smtp server defined, always use that instead of the endpoint
+    # we may make this configurable in the future, but this almost always makes sense
+    endpoint = None
+    if not sconfig.get('smtpserver'):
+        endpoint = config.get('send-endpoint-web', '')
+        if not re.search(r'^https?://', endpoint):
+            logger.debug('Endpoint does not start with https, ignoring: %s', endpoint)
+            endpoint = None
+        if not endpoint:
+            # Use the default endpoint if we are in the kernel repo
+            topdir = b4.git_get_toplevel()
+            if os.path.exists(os.path.join(topdir, 'Kconfig')):
+                logger.debug('No sendemail configs found, will use the default web endpoint')
+                endpoint = DEFAULT_ENDPOINT
 
     # Give the user the last opportunity to bail out
     if not cmdargs.dryrun:
@@ -1821,32 +1828,19 @@ def auto_to_cc() -> None:
             logger.debug('added %s to seen', ltr.addr[1])
             extras.append(ltr)
 
-    tos, ccs, tag_msg, patches = get_prep_branch_as_patches()
+    try:
+        tos, ccs, tag_msg, patches = get_prep_branch_as_patches()
+    except RuntimeError:
+        logger.info('No commits in branch')
+        return
+
     logger.info('Collecting To/Cc addresses')
     # Go through the messages to make to/cc headers
     for commit, msg in patches:
-        if not msg:
-            continue
-        payload = msg.get_payload(decode=True).decode()
-        parts = b4.LoreMessage.get_body_parts(payload)
-        for ltr in parts[2]:
-            if ltr.lname == 'cc':
-                # We treat Cc: in individual patches differently from all other trailers:
-                # they only receive individual patches, not the entire series.
-                continue
-            if not ltr.addr:
-                continue
-            if ltr.addr[1] in seen:
-                continue
-            seen.add(ltr.addr[1])
-            logger.debug('added %s to seen', ltr.addr[1])
-            # Make it a Cc: trailer
-            ltr.name = 'Cc'
-            extras.append(ltr)
-
-        if not commit:
+        if not msg or not commit:
             continue
 
+        logger.debug('Collecting from: %s', msg.get('subject'))
         msgbytes = msg.as_bytes()
         for tname, pairs in (('To', get_addresses_from_cmd(tocmd, msgbytes)),
                              ('Cc', get_addresses_from_cmd(cccmd, msgbytes))):
@@ -1854,6 +1848,7 @@ def auto_to_cc() -> None:
                 if pair[1] not in seen:
                     seen.add(pair[1])
                     ltr = b4.LoreTrailer(name=tname, value=b4.format_addrs([pair]))
+                    logger.debug('  => %s', ltr.as_string())
                     extras.append(ltr)
 
     if not extras:
@@ -1895,12 +1890,6 @@ def cmd_prep(cmdargs: argparse.Namespace) -> None:
         logger.critical('          Stash or commit them first.')
         sys.exit(1)
 
-    if cmdargs.edit_cover:
-        return edit_cover()
-
-    if cmdargs.auto_to_cc:
-        return auto_to_cc()
-
     if cmdargs.reroll:
         msgid = cmdargs.reroll
         msgs = b4.get_pi_thread_by_msgid(msgid, onlymsgids={msgid}, nocache=True)
@@ -1927,23 +1916,30 @@ def cmd_prep(cmdargs: argparse.Namespace) -> None:
     if cmdargs.show_info:
         return show_info()
 
-    if cmdargs.force_revision:
-        return force_revision(cmdargs.force_revision)
-
     if cmdargs.format_patch:
         return format_patch(cmdargs.format_patch)
 
     if cmdargs.compare_to:
         return compare(cmdargs.compare_to)
 
+    if cmdargs.enroll_base or cmdargs.new_series_name:
+        if is_prep_branch():
+            logger.critical('CRITICAL: This appears to already be a b4-prep managed branch.')
+            sys.exit(1)
+
+        start_new_series(cmdargs)
+
+    if cmdargs.force_revision:
+        force_revision(cmdargs.force_revision)
+
     if cmdargs.set_prefixes:
-        return set_prefixes(cmdargs.set_prefixes)
+        set_prefixes(cmdargs.set_prefixes)
 
-    if is_prep_branch():
-        logger.critical('CRITICAL: This appears to already be a b4-prep managed branch.')
-        sys.exit(1)
+    if cmdargs.auto_to_cc:
+        auto_to_cc()
 
-    return start_new_series(cmdargs)
+    if cmdargs.edit_cover:
+        return edit_cover()
 
 
 def cmd_trailers(cmdargs: argparse.Namespace) -> None:
