@@ -17,17 +17,13 @@ import json
 import fnmatch
 import shutil
 import pathlib
-import tempfile
 import io
 import shlex
 import argparse
 
-import urllib.parse
-import xml.etree.ElementTree
-
 import b4
 
-from typing import Optional, Tuple
+from typing import Optional, List
 from string import Template
 
 logger = b4.logger
@@ -42,7 +38,7 @@ Link: ${midurl}
 """
 
 
-def make_am(msgs, cmdargs, msgid):
+def make_am(msgs: List[email.message.Message], cmdargs: argparse.Namespace, msgid: str) -> None:
     config = b4.get_main_config()
     outdir = cmdargs.outdir
     if outdir == '-':
@@ -114,22 +110,6 @@ def make_am(msgs, cmdargs, msgid):
         logger.critical('Total patches: %s', len(am_msgs))
     else:
         logger.info('Total patches: %s (cherrypicked: %s)', len(am_msgs), cmdargs.cherrypick)
-    # Check if any of the followup-trailers is an Obsoleted-by
-    if not cmdargs.checknewer:
-        warned = False
-        for lmsg in lser.patches:
-            # Only check cover letter or first patch
-            if not lmsg or lmsg.counter > 1:
-                continue
-            for ltr in list(lmsg.followup_trailers):
-                if ltr.lname == 'obsoleted-by':
-                    lmsg.followup_trailers.remove(ltr)
-                    if warned:
-                        continue
-                    logger.critical('---')
-                    logger.critical('WARNING: Found an Obsoleted-by follow-up trailer!')
-                    logger.critical('         Rerun with -c to automatically retrieve the new series.')
-                    warned = True
 
     if lser.has_cover and lser.patches[0].followup_trailers and not covertrailers:
         # Warn that some trailers were sent to the cover letter
@@ -213,11 +193,11 @@ def make_am(msgs, cmdargs, msgid):
             if save_maildir:
                 b4.save_maildir(am_msgs, am_filename)
             else:
-                with open(am_filename, 'w') as fh:
+                with open(am_filename, 'wb') as fh:
                     b4.save_git_am_mbox(am_msgs, fh)
         else:
             am_cover = None
-            b4.save_git_am_mbox(am_msgs, sys.stdout)
+            b4.save_git_am_mbox(am_msgs, sys.stdout.buffer)
 
         if lser.has_cover and not cmdargs.nocover:
             lser.save_cover(am_cover)
@@ -240,6 +220,14 @@ def make_am(msgs, cmdargs, msgid):
         if matches:
             base_commit = matches.groups()[0]
 
+    if base_commit and topdir:
+        # Does it actually exist in this tree?
+        if not b4.git_commit_exists(topdir, base_commit):
+            logger.info(' Base: base-commit %s not known, ignoring', base_commit)
+            base_commit = None
+        else:
+            logger.info(' Base: using specified base-commit %s', base_commit)
+
     if not base_commit and topdir and cmdargs.guessbase:
         logger.critical(' Base: attempting to guess base-commit...')
         try:
@@ -259,9 +247,9 @@ def make_am(msgs, cmdargs, msgid):
         if not topdir:
             logger.critical('Could not figure out where your git dir is, cannot shazam.')
             sys.exit(1)
-        ifh = io.StringIO()
+        ifh = io.BytesIO()
         b4.save_git_am_mbox(am_msgs, ifh)
-        ambytes = ifh.getvalue().encode()
+        ambytes = ifh.getvalue()
         if not cmdargs.makefetchhead:
             amflags = config.get('shazam-am-flags', '')
             sp = shlex.shlex(amflags, posix=True)
@@ -374,7 +362,11 @@ def make_am(msgs, cmdargs, msgid):
         mergeflags = config.get('shazam-merge-flags', '--signoff')
         sp = shlex.shlex(mergeflags, posix=True)
         sp.whitespace_split = True
-        mergeargs = ['merge', '--no-ff', '-F', mmf, '--edit', 'FETCH_HEAD'] + list(sp)
+        if cmdargs.no_interactive:
+            edit = '--no-edit'
+        else:
+            edit = '--edit'
+        mergeargs = ['merge', '--no-ff', '-F', mmf, edit, 'FETCH_HEAD'] + list(sp)
         mergecmd = ['git'] + mergeargs
 
         thanks_record_am(lser, cherrypick=cherrypick)
@@ -388,12 +380,17 @@ def make_am(msgs, cmdargs, msgid):
                     sys.exit(130)
             else:
                 logger.info('Invoking: %s', ' '.join(mergecmd))
+            if hasattr(sys, '_running_in_pytest'):
+                # Don't execvp, as this kills our tests
+                out, logstr = b4.git_run_command(None, mergeargs)
+                sys.exit(out)
+
             # We exec git-merge and let it take over
             os.execvp(mergecmd[0], mergecmd)
 
         logger.info('You can now merge or checkout FETCH_HEAD')
         logger.info('  e.g.: %s', ' '.join(mergecmd))
-        return
+        sys.exit(0)
 
     if not base_commit:
         checked, mismatches = lser.check_applies_clean(topdir, at=cmdargs.guessbranch)
@@ -410,7 +407,7 @@ def make_am(msgs, cmdargs, msgid):
     thanks_record_am(lser, cherrypick=cherrypick)
 
 
-def thanks_record_am(lser, cherrypick=None):
+def thanks_record_am(lser: b4.LoreSeries, cherrypick: bool = None) -> None:
     # Are we tracking this already?
     datadir = b4.get_data_dir()
     slug = lser.get_slug(extended=True)
@@ -453,8 +450,16 @@ def thanks_record_am(lser, cherrypick=None):
         logger.debug('All patches missing, not tracking for thanks')
         return
 
-    allto = email.utils.getaddresses([str(x) for x in lmsg.msg.get_all('to', [])])
-    allcc = email.utils.getaddresses([str(x) for x in lmsg.msg.get_all('cc', [])])
+    try:
+        allto = email.utils.getaddresses([str(x) for x in lmsg.msg.get_all('to', [])])
+    except Exception as ex:  # noqa
+        allto = []
+        logger.debug('Unable to parse the To: header in %s: %s', lmsg.msgid, str(ex))
+    try:
+        allcc = email.utils.getaddresses([str(x) for x in lmsg.msg.get_all('cc', [])])
+    except Exception as ex:  # noqa
+        allcc = []
+        logger.debug('Unable to parse the Cc: header in %s: %s', lmsg.msgid, str(ex))
 
     # TODO: check for reply-to and x-original-from
     out = {
@@ -481,7 +486,7 @@ def thanks_record_am(lser, cherrypick=None):
         b4.patchwork_set_state(msgids, pwstate)
 
 
-def save_as_quilt(am_msgs, q_dirname):
+def save_as_quilt(am_msgs: List[email.message.Message], q_dirname: str) -> None:
     if os.path.exists(q_dirname):
         logger.critical('ERROR: Directory %s exists, not saving quilt patches', q_dirname)
         return
@@ -511,30 +516,32 @@ def save_as_quilt(am_msgs, q_dirname):
             sfh.write('%s\n' % patch_filename)
 
 
-def get_extra_series(msgs: list, direction: int = 1, wantvers: Optional[int] = None, nocache: bool = False,
-                     useproject: Optional[str] = None) -> list:
+def get_extra_series(msgs: list, direction: int = 1, wantvers: Optional[int] = None,
+                     nocache: bool = False) -> List[email.message.Message]:
     base_msg = None
     latest_revision = None
     seen_msgids = set()
     seen_covers = set()
-    obsoleted = list()
+    queries = set()
     for msg in msgs:
         msgid = b4.LoreMessage.get_clean_msgid(msg)
         seen_msgids.add(msgid)
         lsub = b4.LoreSubject(msg['Subject'])
-        if direction > 0 and lsub.reply:
-            # Does it have an Obsoleted-by: trailer?
-            rmsg = b4.LoreMessage(msg)
-            trailers, mismatches = rmsg.get_trailers()
-            for ltr in trailers:
-                if ltr.lname == 'obsoleted-by':
-                    for chunk in ltr.value.split('/'):
-                        if chunk.find('@') > 0 and chunk not in seen_msgids:
-                            obsoleted.append(chunk)
-                            break
+
         # Ignore patches above 1
         if lsub.counter > 1:
             continue
+
+        if not lsub.reply:
+            payload = msg.get_payload(decode=True)
+            if isinstance(payload, bytes):
+                payload = payload.decode()
+                matches = re.search(r'^change-id:\s+(\S+)', payload, flags=re.I | re.M)
+                if matches:
+                    logger.debug('Found change-id %s', matches.groups()[0])
+                    q = 'nq:"change-id: %s"' % matches.groups()[0]
+                    queries.add(q)
+
         if base_msg is not None:
             logger.debug('Current base_msg: %s', base_msg['Subject'])
         logger.debug('Checking the subject on %s', lsub.full_subject)
@@ -548,193 +555,121 @@ def get_extra_series(msgs: list, direction: int = 1, wantvers: Optional[int] = N
                 # A patch/series without a cover letter
                 base_msg = msg
 
-    if base_msg is None:
-        logger.debug('Could not find cover of 1st patch in mbox')
+    if not queries and base_msg is None:
         return msgs
 
-    config = b4.get_main_config()
-    loc = urllib.parse.urlparse(config['midmask'])
-    if not useproject:
-        useproject = 'all'
-
-    listarc = f'{loc.scheme}://{loc.netloc}/{useproject}/'
-    # Make sure it exists
-    queryurl = f'{listarc}_/text/config/raw'
-    session = b4.get_requests_session()
-    resp = session.get(queryurl)
-    if not resp.status_code == 200:
-        logger.info('Unable to figure out list archive location')
+    # Get subject info from base_msg again
+    lsub = b4.LoreSubject(base_msg['Subject'])
+    if not len(lsub.prefixes):
+        logger.debug('Not checking for new revisions: no prefixes on the cover letter.')
         return msgs
+    if direction < 0 and latest_revision <= 1:
+        logger.debug('This is the earliest version of the series')
+        return msgs
+    if direction < 0 and wantvers is None:
+        wantvers = [latest_revision - 1]
 
-    nt_msgs = list()
-    if len(obsoleted):
-        for nt_msgid in obsoleted:
-            logger.info('Obsoleted-by: %s', nt_msgid)
-            # Grab this thread from remote
-            t_mbx_url = '%s/%s/t.mbox.gz' % (listarc.rstrip('/'), nt_msgid)
-            potentials = b4.get_pi_thread_by_url(t_mbx_url, nocache=nocache)
-            if potentials:
-                potentials = b4.get_strict_thread(potentials, nt_msgid)
-                nt_msgs += potentials
-                logger.info('   Added %s messages from that thread', len(potentials))
-            else:
-                logger.info('   No messages added from that thread')
-
+    fromeml = email.utils.getaddresses(base_msg.get_all('from', []))[0][1]
+    msgdate = email.utils.parsedate_tz(str(base_msg['Date']))
+    q = '(s:"%s" AND f:"%s")' % (lsub.subject.replace('"', ''), fromeml)
+    queries.add(q)
+    startdate = time.strftime('%Y%m%d', msgdate[:9])
+    if direction > 0:
+        logger.critical('Checking for newer revisions')
+        datelim = 'd:%s..' % startdate
     else:
-        # Get subject info from base_msg again
-        lsub = b4.LoreSubject(base_msg['Subject'])
-        if not len(lsub.prefixes):
-            logger.debug('Not checking for new revisions: no prefixes on the cover letter.')
-            return msgs
-        if direction < 0 and latest_revision <= 1:
-            logger.debug('This is the latest version of the series')
-            return msgs
-        if direction < 0 and wantvers is None:
-            wantvers = [latest_revision - 1]
+        logger.critical('Checking for older revisions')
+        datelim = 'd:..%s' % startdate
 
-        base_msgid = b4.LoreMessage.get_clean_msgid(base_msg)
-        fromeml = email.utils.getaddresses(base_msg.get_all('from', []))[0][1]
-        msgdate = email.utils.parsedate_tz(str(base_msg['Date']))
-        startdate = time.strftime('%Y%m%d', msgdate[:9])
-        if direction > 0:
-            q = 's:"%s" AND f:"%s" AND d:%s..' % (lsub.subject.replace('"', ''), fromeml, startdate)
-            queryurl = '%s?%s' % (listarc, urllib.parse.urlencode({'q': q, 'x': 'A', 'o': '-1'}))
-            logger.critical('Checking for newer revisions on %s', listarc)
-        else:
-            q = 's:"%s" AND f:"%s" AND d:..%s' % (lsub.subject.replace('"', ''), fromeml, startdate)
-            queryurl = '%s?%s' % (listarc, urllib.parse.urlencode({'q': q, 'x': 'A', 'o': '1'}))
-            logger.critical('Checking for older revisions on %s', listarc)
+    q = '(%s) AND %s' % (' OR '.join(queries), datelim)
+    q_msgs = b4.get_pi_search_results(q, nocache=nocache)
+    if not q_msgs:
+        return msgs
 
-        logger.debug('Query URL: %s', queryurl)
-        session = b4.get_requests_session()
-        resp = session.get(queryurl)
-        # try to parse it
-        try:
-            tree = xml.etree.ElementTree.fromstring(resp.content)
-        except xml.etree.ElementTree.ParseError as ex:
-            logger.debug('Unable to parse results, ignoring: %s', ex)
-            resp.close()
-            return msgs
-        resp.close()
-        ns = {'atom': 'http://www.w3.org/2005/Atom'}
-        entries = tree.findall('atom:entry', ns)
-        seen_urls = set()
-
-        for entry in entries:
-            title = entry.find('atom:title', ns).text
-            lsub = b4.LoreSubject(title)
-            if lsub.reply or lsub.counter > 1:
-                logger.debug('Ignoring result (not interesting): %s', title)
-                continue
-            link = entry.find('atom:link', ns).get('href')
-            if direction > 0 and lsub.revision <= latest_revision:
-                logger.debug('Ignoring result (not new revision): %s', title)
-                continue
-            elif direction < 0 and lsub.revision >= latest_revision:
-                logger.debug('Ignoring result (not old revision): %s', title)
-                continue
-            elif direction < 0 and lsub.revision not in wantvers:
-                logger.debug('Ignoring result (not revision we want): %s', title)
-                continue
-            if link.find('/%s/' % base_msgid) > 0:
-                logger.debug('Ignoring result (same thread as ours):%s', title)
-                continue
-            if lsub.revision == 1 and lsub.revision == latest_revision:
-                # Someone sent a separate message with an identical title but no new vX in the subject line
-                if direction > 0:
-                    # It's *probably* a new revision.
-                    logger.debug('Likely a new revision: %s', title)
-                else:
-                    # It's *probably* an older revision.
-                    logger.debug('Likely an older revision: %s', title)
-            elif direction > 0 and lsub.revision > latest_revision:
-                logger.debug('Definitely a new revision [v%s]: %s', lsub.revision, title)
-            elif direction < 0 and lsub.revision < latest_revision:
-                logger.debug('Definitely an older revision [v%s]: %s', lsub.revision, title)
-            else:
-                logger.debug('No idea what this is: %s', title)
-                continue
-            t_mbx_url = '%st.mbox.gz' % link
-            if t_mbx_url in seen_urls:
-                continue
-            seen_urls.add(t_mbx_url)
-            logger.info('New revision: %s', title)
-            potentials = b4.get_pi_thread_by_url(t_mbx_url, nocache=nocache)
-            if potentials:
-                nt_msgs += potentials
-                logger.info('   Added %s messages from that thread', len(potentials))
-
-    # Append all of these to the existing mailbox
-    for nt_msg in nt_msgs:
-        nt_msgid = b4.LoreMessage.get_clean_msgid(nt_msg)
-        if nt_msgid in seen_msgids:
-            logger.debug('Duplicate message, skipping')
+    seen_revisions = dict()
+    for q_msg in q_msgs:
+        q_msgid = b4.LoreMessage.get_clean_msgid(q_msg)
+        lsub = b4.LoreSubject(q_msg.get('subject'))
+        if q_msgid in seen_msgids:
+            logger.debug('Skipping %s: already have it', lsub.full_subject)
             continue
-        nt_subject = re.sub(r'\s+', ' ', nt_msg['Subject'])
-        logger.debug('Adding: %s', nt_subject)
-        msgs.append(nt_msg)
-        seen_msgids.add(nt_msgid)
+        if lsub.reply:
+            # These will get sorted out later
+            logger.debug('Adding reply: %s', lsub.full_subject)
+            msgs.append(q_msg)
+            seen_msgids.add(q_msgid)
+            continue
+
+        if direction > 0 and lsub.revision <= latest_revision:
+            logger.debug('Ignoring result (not new revision): %s', lsub.full_subject)
+            continue
+        elif direction < 0 and lsub.revision >= latest_revision:
+            logger.debug('Ignoring result (not old revision): %s', lsub.full_subject)
+            continue
+        elif direction < 0 and lsub.revision not in wantvers:
+            logger.debug('Ignoring result (not revision we want): %s', lsub.full_subject)
+            continue
+
+        if lsub.revision == 1 and lsub.revision == latest_revision:
+            # Someone sent a separate message with an identical title but no new vX in the subject line
+            if direction > 0:
+                # It's *probably* a new revision.
+                logger.debug('Likely a new revision: %s', lsub.full_subject)
+            else:
+                # It's *probably* an older revision.
+                logger.debug('Likely an older revision: %s', lsub.full_subject)
+        elif direction > 0 and lsub.revision > latest_revision:
+            logger.debug('Definitely a new revision [v%s]: %s', lsub.revision, lsub.full_subject)
+        elif direction < 0 and lsub.revision < latest_revision:
+            logger.debug('Definitely an older revision [v%s]: %s', lsub.revision, lsub.full_subject)
+        else:
+            logger.debug('No idea what this is: %s', lsub.subject)
+            continue
+        if lsub.revision not in seen_revisions:
+            seen_revisions[lsub.revision] = 0
+        seen_revisions[lsub.revision] += 1
+        logger.debug('Adding: %s', lsub.full_subject)
+        msgs.append(q_msg)
+        seen_msgids.add(q_msgid)
+
+    for rev, count in seen_revisions.items():
+        logger.info('  Added from v%s: %s patches', rev, count)
 
     return msgs
 
 
-def get_msgs(cmdargs: argparse.Namespace) -> Tuple[Optional[str], Optional[list]]:
-    msgid = None
-    if not cmdargs.localmbox:
-        msgid = b4.get_msgid(cmdargs)
-        if not msgid:
-            logger.error('Error: pipe a message or pass msgid as parameter')
-            sys.exit(1)
+def is_maildir(dest: str) -> bool:
+    if (os.path.isdir(os.path.join(dest, 'new'))
+            and os.path.isdir(os.path.join(dest, 'cur'))
+            and os.path.isdir(os.path.join(dest, 'tmp'))):
+        return True
+    return False
 
-        pickings = set()
-        if 'cherrypick' in cmdargs and cmdargs.cherrypick == '_':
-            # Just that msgid, please
-            pickings = {msgid}
-        msgs = b4.get_pi_thread_by_msgid(msgid, useproject=cmdargs.useproject, nocache=cmdargs.nocache,
-                                         onlymsgids=pickings)
-        if not msgs:
-            return None, msgs
+
+def refetch(dest: str) -> None:
+    if is_maildir(dest):
+        mbox = mailbox.Maildir(dest)
     else:
-        if cmdargs.localmbox == '-':
-            # The entire mbox is passed via stdin, so mailsplit it and use the first message for our msgid
-            with tempfile.TemporaryDirectory() as tfd:
-                msgs = b4.mailsplit_bytes(sys.stdin.buffer.read(), tfd)
-            if not len(msgs):
-                logger.critical('Stdin did not contain any messages')
-                sys.exit(1)
+        mbox = mailbox.mbox(dest)
 
-        elif os.path.exists(cmdargs.localmbox):
-            msgid = b4.get_msgid(cmdargs)
-            if os.path.isdir(cmdargs.localmbox):
-                in_mbx = mailbox.Maildir(cmdargs.localmbox)
-            else:
-                in_mbx = mailbox.mbox(cmdargs.localmbox)
-
-            if msgid:
-                msgs = b4.get_strict_thread(in_mbx, msgid)
-                if not msgs:
-                    logger.critical('Could not find %s in %s', msgid, cmdargs.localmbox)
-                    sys.exit(1)
-            else:
-                msgs = in_mbx
+    by_msgid = dict()
+    for key, msg in mbox.items():
+        msgid = b4.LoreMessage.get_clean_msgid(msg)
+        if msgid not in by_msgid:
+            amsgs = b4.get_pi_thread_by_msgid(msgid, nocache=True)
+            for amsg in amsgs:
+                amsgid = b4.LoreMessage.get_clean_msgid(amsg)
+                if amsgid not in by_msgid:
+                    by_msgid[amsgid] = amsg
+        if msgid in by_msgid:
+            mbox.update(((key, by_msgid[msgid]),))
+            logger.info('Refetched: %s', msg.get('Subject'))
         else:
-            logger.critical('Mailbox %s does not exist', cmdargs.localmbox)
-            sys.exit(1)
-
-    if msgid and 'noparent' in cmdargs and cmdargs.noparent:
-        msgs = b4.get_strict_thread(msgs, msgid, noparent=True)
-
-    if not msgid and msgs:
-        for msg in msgs:
-            msgid = msg.get('Message-ID', None)
-            if msgid:
-                msgid = msgid.strip('<>')
-                break
-
-    return msgid, msgs
+            logger.warn('WARNING: Message-id not known: %s', msgid)
+    mbox.close()
 
 
-def main(cmdargs):
+def main(cmdargs: argparse.Namespace) -> None:
     if cmdargs.subcmd == 'shazam':
         # We force some settings
         cmdargs.checknewer = True
@@ -753,12 +688,20 @@ def main(cmdargs):
         # Force nocache mode
         cmdargs.nocache = True
 
-    msgid, msgs = get_msgs(cmdargs)
-    if not msgs:
-        return
+    if cmdargs.subcmd == 'mbox' and cmdargs.refetch:
+        return refetch(cmdargs.refetch)
 
-    if len(msgs) and cmdargs.checknewer:
-        msgs = get_extra_series(msgs, direction=1, useproject=cmdargs.useproject)
+    try:
+        msgid, msgs = b4.retrieve_messages(cmdargs)
+    except LookupError as ex:
+        logger.critical('CRITICAL: %s', ex)
+        sys.exit(1)
+
+    if not msgs:
+        sys.exit(1)
+
+    if len(msgs) and cmdargs.checknewer and b4.can_network:
+        msgs = get_extra_series(msgs, direction=1, nocache=cmdargs.nocache)
 
     if cmdargs.subcmd in ('am', 'shazam'):
         make_am(msgs, cmdargs, msgid)
@@ -767,13 +710,11 @@ def main(cmdargs):
     logger.info('%s messages in the thread', len(msgs))
     if cmdargs.outdir == '-':
         logger.info('---')
-        b4.save_git_am_mbox(msgs, sys.stdout)
+        b4.save_mboxrd_mbox(msgs, sys.stdout.buffer, mangle_from=False)
         return
 
     # Check if outdir is a maildir
-    if (os.path.isdir(os.path.join(cmdargs.outdir, 'new'))
-            and os.path.isdir(os.path.join(cmdargs.outdir, 'cur'))
-            and os.path.isdir(os.path.join(cmdargs.outdir, 'tmp'))):
+    if is_maildir(cmdargs.outdir):
         mdr = mailbox.Maildir(cmdargs.outdir)
         have_msgids = set()
         added = 0
@@ -811,7 +752,7 @@ def main(cmdargs):
         logger.info('Saved maildir %s', savename)
         return
 
-    with open(savename, 'w') as fh:
-        b4.save_git_am_mbox(msgs, fh)
+    with open(savename, 'wb') as fh:
+        b4.save_mboxrd_mbox(msgs, fh, mangle_from=True)
 
     logger.info('Saved %s', savename)
